--[[
This file is executed in sandbox.
User is encouraged to implement his own algorithm how to move paddle.

Basic requierement for this file is that it return two function
that will drive each paddle. Function should return 1 if paddle
need to be moved right, -1 to move paddle left and 0 not to move paddle.

Paddle1 is on top of the screen and paddle2 is on the bottom.
These function are called with restricted enviroment.
In the enviroment there is table named 'world' that holds
current state of ball, both paddles and dimensions of playground.

It looks like this
         world = { ball = {}, paddle1 = {}, paddle2 = {}, playground = {} }
playground hold area where ball can  move. It has following keys
x1, x2, y1, y2.

ball, paddle1, and paddle2 holds info about coresponding object.
it has following keys:
{ x, y -- position of upper left corner
  w, h -- width and height of object
  direction -- in radian 
  velocity }

Look bellow how these data are used
]]

local function donot_move()
   return 0;
end

local function goto_center(paddle)
   local center = (world.playground.x1 + world.playground.x2)/2
   local diff = center - (paddle.x + paddle.w/2)
   if (diff > 0) then
      return 1
   elseif (diff < 0) then
      return -1
   else
      return 0;
   end
end

local function follow_ball(paddle)
   local ball = world.ball
   ball_center = ball.x+(ball.w/2)
   paddle_center = paddle.x + (paddle.w/2)
   local diff = ball_center - paddle_center
   if (diff > 0) then
      return 1
   elseif (diff < 0) then
      return -1
   else
      return 0;
   end
end

local function computer_move_paddle1()
   if world.ball.direction > 0  then
      -- ball is moving away from paddle
      return goto_center(world.paddle1)
   else
      return follow_ball(world.paddle1)
   end
end

local function computer_move_paddle2()
   if world.ball.direction < 0  then
      return donot_move()
   else
      return follow_ball(world.paddle2)
   end
end

return computer_move_paddle1, computer_move_paddle2
