game_engine = newclass("game_engine")

local function get_configuration(conf_file)
   local env = {}
   env.math = copy_table(math)
   env.math.randomseed = nil
   local ret, conf =  dofile_in_sandbox(conf_file, env)
   if not ret  then
      print(conf)
      conf = {}
   end
   if ret and (type(conf) ~= "table") then
      print("Error in configuretion file. You need to return table!")
      conf = {}
   end
   -- check sanity of the supplied values and use defaults if needed
   -- todo
   -- instead of only checking if valeues are present
   -- I realy need to check if valuews are sane/valid
   nc = {}
   nc.menu_image = conf.menu_image or "data/menu.bmp"
   nc.end1_image = conf.end1_image or "data/end1.bmp"
   nc.end2_image = conf.end2_image or "data/end2.bmp"
   nc.ball_image = conf.ball_image or "data/ball.bmp"
   nc.paddle_small_image = conf.paddle_small_image or "data/paddle_small.bmp"
   nc.paddle_medium_image = conf.paddle_medium_image or "data/paddle_medium.bmp"
   nc.paddle_big_image = conf.paddle_big_image or "data/paddle_big.bmp"
   nc.paddle1 = conf.paddle1 or "normal"
   nc.paddle2 = conf.paddle2 or "normal"
   nc.paddle1 = string.lower(nc.paddle1)
   nc.paddle2 = string.lower(nc.paddle2)

   if conf.computer_player1==true then nc.computer_player1=true else nc.computer_player1 = false end
   if conf.computer_player2==true then nc.computer_player2=true else nc.computer_player2 = false end

   nc.caption = conf.caption or "Pong"
   nc.screen_width = conf.screen_width or 320
   nc.screen_height = conf.screen_height or 240
   nc.screen_bpp = conf.screen_bpp or 16

   nc.acceleration = conf.acceleration or 1.003
   nc.fsp = conf.fsp or 30
   nc.paddle_velocity = conf.paddle_velocity or 180/nc.fsp
   nc.ball_velocity = conf.ball_velocity or nc.paddle_velocity * 1.3
   nc.paddle_flatness = conf.paddle_flatness or 100
   nc.slope = conf.slope or math.pi/8
   nc.sdl_flags = conf.sdl_flags or SDL_INIT_EVERYTHING
   nc.video_flags = conf.video_flags or SDL_SWSURFACE
   return nc
end

function game_engine:start()
   self.world = {}  -- this is where all game variables are
   self.mode_stack = {}  -- stack of modes
   self.conf = {} -- configuration
   self.quit = false

   -- parse the configuration file
   self.conf = get_configuration("game_cfg.lua")
   local sc = self.conf
   SDL.SDL_Init(sc.sdl_flags)
   SDL.SDL_WM_SetCaption(sc.caption, sc.caption)
   -- create window
   self.screen = SDL.SDL_SetVideoMode(sc.screen_width,
                                   sc.screen_height,
                                   sc.screen_bpp,
                                   sc.video_flags)
   SDL.SDL_ShowCursor(SDL_DISABLE)
   -- locading graphics
   self.pics = {}
   self.pics.menu = load_bmp(sc.menu_image)
   self.pics.end1 = load_bmp(sc.end1_image)
   self.pics.end2 = load_bmp(sc.end2_image)
   self.pics.ball = load_bmp(sc.ball_image)
   self.pics.paddle_small = load_bmp(sc.paddle_small_image)
   self.pics.paddle_medium = load_bmp(sc.paddle_medium_image)
   self.pics.paddle_big = load_bmp(sc.paddle_big_image)

   if nc.paddle1 == "big" then
      self.pics.paddle1 = self.pics.paddle_big
   elseif nc.paddle1 == "small" then
      self.pics.paddle1 = self.pics.paddle_small
   else
      self.pics.paddle1 = self.pics.paddle_medium
   end
   if nc.paddle2 == "big" then
      self.pics.paddle2 = self.pics.paddle_big
   elseif nc.paddle2 == "small" then
      self.pics.paddle2 = self.pics.paddle_small
   else
      self.pics.paddle2 = self.pics.paddle_medium
   end
end

function game_engine:stop()
   local old_mode
   -- cleanuo all reminding modes on the stack
   while (#self.mode_stack > 0) do
      old_mode = table.remove(self.mode_stack)
      old_mode:stop(self)
   end

   self.pics.paddle1=nil
   self.pics.paddle2=nil

   for k, picture in pairs(self.pics) do
         SDL.SDL_FreeSurface(picture)
   end
   SDL.SDL_ShowCursor(SDL_ENABLE)
   SDL.SDL_Quit()
   self.quit = true
end


function game_engine:change_mode(mode)
   if (#self.mode_stack > 0) then
      local old_mode = table.remove(self.mode_stack)
      old_mode:stop(self)
   end
   self.mode_stack[#self.mode_stack + 1] = mode
   mode:start(self)
end

function game_engine:push_mode(mode)
   if (#self.mode_stack > 0) then
      self.mode_stack[#self.mode_stack]:pause(self)
   end
   self.mode_stack[#self.mode_stack + 1] = mode
   mode:start(self)
end

function game_engine:pop_mode()
   if (#self.mode_stack > 0) then
      local old_mode = table.remove(self.mode_stack)
      old_mode:stop(self)
   end
   if (#self.mode_stack > 0) then
      self.mode_stack[#self.mode_stack]:resume(self)
   end
end

function game_engine:handle_events()
   if (#self.mode_stack > 0) then
      local event=alien.buffer()
      while SDL.SDL_PollEvent(event)==1 do
         self.mode_stack[#self.mode_stack]:handle_events(self, event)
      end
   else
      self.quit = true
   end
end

function game_engine:update()
   if (#self.mode_stack > 0) then
      self.mode_stack[#self.mode_stack]:update(self)
   else
      self.quit = true
   end
end

function game_engine:render()
   if (#self.mode_stack > 0) then
      self.mode_stack[#self.mode_stack]:render(self)
   else
      self.quit = true
   end
end
