-- load and conver image to display format
function load_bmp(file)
   local temp = SDL_LoadBMP(file)
   local bg = SDL.SDL_DisplayFormat(temp)
   SDL.SDL_FreeSurface(temp);
   return bg
end

-- execute  file in sandbox with only math functions available
function dofile_in_sandbox(file, env)
   local fun, message = loadfile(file)
   if not fun then return nil, message end
   setfenv(fun, env)
   return pcall(fun)
end

-- shallow copy of table
function copy_table(t)
   local nt = {}
   for k, v in pairs(t) do nt[k] = v end
   return nt
end

-- as described at http://lua-users.org/wiki/SandBoxes
function create_sandbox()
   local env = {}
   env._G = env
   local safe = { 'assert', 'ipairs', 'pairs', 'next', 'pcall', 'print',
      'tonumber', 'tostring', 'type', 'unpack', '_VERSION', 'xpcall' }
   for _, v in ipairs(safe) do
      env[v] = _G[v]
   end
   -- omitted because I do not handle coroutine.yield
   --   env.coroutine = copy_table(coroutine)
   env.string = copy_table(string)
   env.string.dump = nil        -- potentialy dangerous
   env.table = copy_table(table)
   env.math = copy_table(math)
   env.math.randomseed = nil

   return env
end
