play_mode = newclass("play_mode", game_mode)
dofile("play_objects.lua")

local env
   
function play_mode:start(game)
   local w = game.world
   local screen_width = game.conf.screen_width
   local screen_height = game.conf.screen_height
   local pics = game.pics
   -- create object in our game
   w.ball = play_objects(0, 0,
                         math.pi/4, 0,
                         pics.ball)
   w.paddle1 = play_objects(0, 0,
                            0, 0,
                            pics.paddle1)
   w.paddle2 = play_objects(0, 0,
                            math.pi, 0,
                            pics.paddle2)
   -- left and right walls
   -- put them just off the screen
   w.wall1 = {x=-20, y=0, w=20, h=screen_height}
   w.wall2 = {x=screen_width, y=0, w=20, h=screen_height}
   -- top and bottom walls
   -- for detectin if ball has gonee behind the paddle
   w.wall3 = {x=-0, y=-w.ball.h/2 - 20, w=screen_width,  h=20}
   w.wall4 = {x=0, y=w.ball.h/2 + screen_height, w=screen_width, h=20}
   -- place paddles in the center
   w.paddle1.x = (screen_width - w.paddle1.w)/2
   w.paddle2.x = (screen_width - w.paddle2.w)/2
   -- bottom of the screen
   w.paddle2.y = screen_height - w.paddle2.h
   -- move ball to left or right
   if math.random(2) == 1 then
      w.ball.direction = math.pi/4
   else
      w.ball.direction = 3*math.pi/4
   end
   -- randomly places ball on the top or bottom of the screen
   w.ball.x = (screen_width - w.ball.w)/2
   if math.random(2) == 1 then
      -- top of the screen
      w.ball.y = w.paddle1.h
   else
      -- bottom of the screen
      w.ball.y = screen_height - w.paddle2.h - w.ball.h
      w.ball.direction = -w.ball.direction
   end

   -- for using user suplied function for moving paddles
   env  = create_sandbox()
   do
      local ai_file = "ai_move_paddle.lua"
      local funcs =  { dofile_in_sandbox(ai_file, env) }
      if not funcs[1]
         or (type(funcs[2]) ~= "function")
         or (type(funcs[3]) ~= "function") then
         comp_move_paddle1 = function () return 0 end
         comp_move_paddle2 = comp_move_paddle1
         print (funcs[2])
      elseif (type(funcs[2]) ~= "function")
         or (type(funcs[3]) ~= "function") then
         comp_move_paddle1 = function () return 0 end
         comp_move_paddle2 = comp_move_paddle1
         print ("You need to return two function from '".. ai_file .."' file")
      else
         comp_move_paddle1, comp_move_paddle2 = funcs[2], funcs[3]
      end
   end

end

function play_mode:stop(game)
end

function play_mode:pause(game)
end

function play_mode:resume(game)
end

function play_mode:handle_events(game, event)
   local etype=event[1]
   if etype == SDL_QUIT then
      -- close button clicked
      game.quit=true
   end
   if etype == SDL_KEYDOWN then
--      local sym=event[9]
      local sym=event:get(9, "int")
      if sym == SDLK_RIGHT then
         --start moving paddle2 to the right
         game.world.paddle2.direction = 0
         game.world.paddle2.velocity = game.conf.paddle_velocity
      elseif sym == SDLK_LEFT then
         --start moving paddle2 to the left
         game.world.paddle2.direction = math.pi
         game.world.paddle2.velocity = game.conf.paddle_velocity
      elseif sym == SDLK_x then
         --start moving paddle1 to the right
         game.world.paddle1.direction = 0
         game.world.paddle1.velocity = game.conf.paddle_velocity
      elseif sym == SDLK_z then
         --start moving paddle1 to the left
         game.world.paddle1.direction = math.pi
         game.world.paddle1.velocity = game.conf.paddle_velocity
      end
   elseif etype == SDL_KEYUP then
      -- handle the keyboard
--      local sym=event[9]
      local sym=event:get(9, "int")
      --      if sym==SDLK_q or sym==SDLK_ESCAPE then
      if sym == SDLK_ESCAPE then
         self:pop_mode(game)
      elseif sym == SDLK_RIGHT then
         --stop moving paddle2 if it is moving to the right
         if math.abs(game.world.paddle2.direction) < 0.001 then
            game.world.paddle2.velocity = 0
         end
      elseif sym == SDLK_LEFT then
         if math.abs(game.world.paddle2.direction - math.pi) < 0.001 then
            game.world.paddle2.velocity = 0
         end
      elseif sym == SDLK_x then
         --stop moving paddle1 if it is moving to the right
         if math.abs(game.world.paddle1.direction) < 0.001 then
            game.world.paddle1.velocity = 0
         end
      elseif sym == SDLK_z then
         if math.abs(game.world.paddle1.direction - math.pi) < 0.001 then
            game.world.paddle1.velocity = 0
         end
      elseif sym == SDLK_SPACE then
         -- start moving ball
         if game.world.ball.velocity == 0 then
            game.world.ball.velocity  = game.conf.ball_velocity
         end
      end
   elseif true then
   end
end

local function bounce_ball(ball, nx, ny)
--     what is dot product??
--     dot product of two vector is scalar defined by
--        a . b = a1*b1 + a2*b2
--     where a=(a1, a2)    b=(b1, b2)
--      if lenght of a and b vectors is 1 |a| = 1 and |b| = 1 then
--      dot product is equal to cos of angle between them
--      it comes from equation
--           a . b = |a| * |b| * cos(alfa)
   local m = math
   local direction = ball.direction
   local dx = m.cos(direction)
   local dy = m.sin(direction)
   local dot = dx * nx + dy * ny
   -- Heh, I do not remember from which notebook I got next two lines
   -- but it works!
   dx = dx - 2 * dot * nx
   dy = dy - 2 * dot * ny
   ball.direction = m.atan2(dy, dx)
end

local function collision(ball, object)
   local btop, otop = ball.y, object.y --top
   local bleft, oleft = ball.x, object.x --left
   local bbottom = btop + ball.h     --bottom (it has coordinate bigger than top)
   local obottom = otop + object.h   --bottom
   local bright = bleft + ball.w       --right
   local oright = oleft + object.w    --right

   -- note that because of how computer see screen coordinates
   -- btop is allways less than bbottom!
   if (btop > obottom) or
      (bleft > oright) or
      (bright < oleft) or
      (bbottom < otop) then
      return false
   else
      return true
   end
end

function play_mode:create_safe_world(game)
   local b = game.world.ball
   local p1 = game.world.paddle1
   local p2 = game.world.paddle2
   local w1 = game.world.wall1
   local w2 = game.world.wall2

   local world = {}
   world.ball = {
      x = b.x, y = b.y,
      w = b.w, h = b.h,
      direction = b.direction,
      velocity = b.velocity }
   world.paddle1 = {
      x = p1.x, y = p1.y,
      w = p1.w, h = p1.h,
      direction = p1.direction,
      velocity = p1.velocity }
   world.paddle2 = {
      x = p2.x, y = p2.y,
      w = p2.w, h = p2.h,
      direction = p2.direction,
      velocity = p2.velocity }
   world.playground = {
      x1 = w1.x + w1.w, x2 = w2.x,
      y1 = w1.y + w1.h, y2 = w2.y + w2.h }
   return world
end

function play_mode:update(game)
   local w = game.world
   local c = game.conf
   local b = w.ball
   local p1 = w.paddle1         --top paddle
   local p2 = w.paddle2         --lower paddle
   local w1 = w.wall1           --left wall
   local w2 = w.wall2           --right wall
   local w3 = w.wall3           --top wall
   local w4 = w.wall4           --bottom wall

   -- move paddles
   if c.computer_player1 then
      env.world = self:create_safe_world(game)
      local direction = comp_move_paddle1()
      if direction == 0 then
         p1.velocity = 0
      elseif direction < 0 then
         p1.direction = math.pi
         p1.velocity = c.paddle_velocity
      else
         p1.direction = 0
         p1.velocity = c.paddle_velocity
      end
   end
   if c.computer_player2 then
      env.world = self:create_safe_world(game)
      local direction = comp_move_paddle2()
      if direction == 0 then
         p2.velocity = 0
      elseif direction < 0 then
         p2.direction = math.pi
         p2.velocity = c.paddle_velocity
      else
         p2.direction = 0
         p2.velocity = c.paddle_velocity
      end
   end
   
   p1:update(game)
   p2:update(game)
   -- check if paddles are gone off the screen
   if (p1.x < w1.x + w1.w) then p1.x = w1.x + w1.w end
   if (p1.x > w2.x - p1.w) then p1.x = w2.x - p1.w end
   if (p2.x < w1.x + w1.w) then p2.x = w1.x + w1.w end
   if (p2.x > w2.x - p2.w) then p2.x = w2.x - p2.w end


   -- move ball
   if b.velocity == 0 then
      if b.y < game.conf.screen_height / 2 then
         -- follow paddle1
         b.x = p1.x + (p1.w - b.w) / 2 
      else
         -- follow paddle2
         b.x = p2.x + (p2.w - b.w) / 2 
      end 
   else
      b:update(game)
   end
   
-- check if ball hit some of paddles or walls
   if collision(b, w1) then
      -- ball hit the left wall
      if (math.abs(b.direction) > math.pi/2) then
         -- todo: play sound here
         local nx = 1
         local ny = 0
         bounce_ball(b, nx, ny)
      end
   end
   if collision(b, w2) then
      -- ball hit the left wall
      if (math.abs(b.direction) < math.pi/2) then
         -- todo: play sound here
         local nx = -1
         local ny = 0
         bounce_ball(b, nx, ny)
      end
   end
   if collision(b, p1) then
      -- ball hit the top paddle
      if (b.direction < 0) then
         -- todo: play sound here
         -- todo: skew the normal (nx, ny) a bit depending on
         -- where did ball hit the paddle
         local nx = 0
         local ny = 1
         bounce_ball(b, nx, ny)
         -- accelerate ball a bit
         b.velocity = b.velocity * c.acceleration
      end
   end
   if collision(b, p2) then
      -- ball hit the top paddle
      if (b.direction > 0) then
         -- todo: play sound here
         local nx = 0
         local ny = -1
         bounce_ball(b, nx, ny)
         -- accelerate ball a bit
         b.velocity = b.velocity * c.acceleration
      end
   end
   if collision(b, w3) then
      -- ball is behind paddle1
      -- player 2 wins
      game.world.winner = 2
      self:change_mode(game, win_mode())
   end
   if collision(b, w4) then
      -- ball is behind paddle2
      -- player 1 wins
      game.world.winner = 1
      self:change_mode(game, win_mode())
   end

end

function play_mode:render(game)
   local buf = alien.buffer(game.screen)
   local format = buf:get(5, "pointer")
   local bg = SDL.SDL_MapRGB(format, 0, 0, 0)
   SDL.SDL_FillRect(game.screen, nil,  bg)
   game.world.ball:render(game)
   game.world.paddle1:render(game)
   game.world.paddle2:render(game)
   -- draw walls
   -- todo
   SDL.SDL_UpdateRect(game.screen, 0,0,0,0)
end
